package ru.bosony.model.maps;

/**
 * @author indvdum (gotoindvdum[at]gmail[dot]com) <br>
 *         15.07.2012 0:28:52
 * 
 */
public enum ExampleMaps {
	/**
	 * <pre>
		######
		#. *R#
		#  \.#
		#\ * #
		L  .\#
		######
	 * </pre>
	 */
	MAP_01(
			"######\n" + 
			"#. *R#\n" + 
			"#  \\.#\n" + 
			"#\\ * #\n" + 
			"L  .\\#\n" + 
			"######"
			),
	/**
	 * <pre>
		#######
		#..***#
		#..\\\#
		#...**#
		#.*.*\#
		LR....#
		#######
	 * </pre>
	 */
	MAP_02(
			"#######\n" + 
			"#..***#\n" + 
			"#..\\\\\\#\n" + 
			"#...**#\n" + 
			"#.*.*\\#\n" + 
			"LR....#\n" + 
			"#######"
			),
	/**
	 * <pre>
		########
		#..R...#
		#..*...#
		#..#...#
		#.\.\..L
		####**.#
		#\.....#
		#\..* .#
		########
	 * </pre>
	 */
	MAP_03(
			"########\n" + 
			"#..R...#\n" + 
			"#..*...#\n" + 
			"#..#...#\n" + 
			"#.\\.\\..L\n" + 
			"####**.#\n" + 
			"#\\.....#\n" + 
			"#\\..* .#\n" + 
			"########"
			),
	/**
	 * <pre>
		#########
		#.*..#\.#
		#.\..#\.L
		#.R .##.#
		#.\  ...#
		#..\  ..#
		#...\  ##
		#....\ \#
		#########
	 * </pre>
	 */
	MAP_04(
			"#########\n" + 
			"#.*..#\\.#\n" + 
			"#.\\..#\\.L\n" + 
			"#.R .##.#\n" + 
			"#.\\  ...#\n" + 
			"#..\\  ..#\n" + 
			"#...\\  ##\n" + 
			"#....\\ \\#\n" + 
			"#########"
			),
	/**
	 * <pre>
		############
		#..........#
		#.....*....#
		#..\\\\\\..#
		#.     ....#
		#..\\\\\\\.#
		#..\..    .#
		#..\.. ....#
		#..... ..* #
		#..### ### #
		#...R#\#\\.#
		######L#####
	 * </pre>
	 */
	MAP_05(
			"############\n" + 
			"#..........#\n" + 
			"#.....*....#\n" + 
			"#..\\\\\\\\\\\\..#\n" + 
			"#.     ....#\n" + 
			"#..\\\\\\\\\\\\\\.#\n" + 
			"#..\\..    .#\n" + 
			"#..\\.. ....#\n" + 
			"#..... ..* #\n" + 
			"#..### ### #\n" + 
			"#...R#\\#\\\\.#\n" + 
			"######L#####"
			),
	/**
	 * <pre>
		###############
		#\\\.......** #
		#\\#.#####...##
		#\\#.....*##. #
		#\#####\...## #
		#\......####* #
		#\.######* #.\#
		#\.#. *...##.##
		#\##. ..  *...#
		#\...... L#.#.#
		###########.#.#
		#\..........#.#
		##.##########.#
		#R.#\.........#
		###############
	 * </pre>
	 */
	MAP_06(
			"###############\n" + 
			"#\\\\\\.......** #\n" + 
			"#\\\\#.#####...##\n" + 
			"#\\\\#.....*##. #\n" + 
			"#\\#####\\...## #\n" + 
			"#\\......####* #\n" + 
			"#\\.######* #.\\#\n" + 
			"#\\.#. *...##.##\n" + 
			"#\\##. ..  *...#\n" + 
			"#\\...... L#.#.#\n" + 
			"###########.#.#\n" + 
			"#\\..........#.#\n" + 
			"##.##########.#\n" + 
			"#R.#\\.........#\n" + 
			"###############"
			),
	/**
	 * <pre>
	    #######
	    ##    *#
	     ##R  *##
	      ##\\\\##
	       ##....##
	      ##..\ . ##
	     ## . L .  ##
	    ##\\\# #\\\\##
	   ######   #######
	 * </pre>
	 */
	MAP_07(
			"    #######\n" + 
			"    ##    *#\n" + 
			"     ##R  *##\n" + 
			"      ##\\\\\\\\##\n" + 
			"       ##....##\n" + 
			"      ##..\\ . ##\n" + 
			"     ## . L .  ##\n" + 
			"    ##\\\\\\# #\\\\\\\\##\n" + 
			"   ######   #######"
			),
	/**
	 * <pre>
		##############
		#\\... ......#
		###.#. ...*..#
		  #.#. ... ..#
		### #.   \ ..#
		#. .#..... **#######
		#.#\#..... ..\\\*. #
		#*\\#.###. ####\\\ #
		#\\.#.     ...## \ #
		#\#.#..... ....# \ #  
		###.#..... ....#   ##
		#\\.#..... ....#\   # 
		########.. ..###*####
		#......... .........#
		#......... ....***..#
		#..\\\\\ # ####.....#
		#........*R..\\\   .#
		##########L##########
	 * </pre>
	 */
	MAP_08(
			"##############\n" + 
			"#\\\\... ......#\n" + 
			"###.#. ...*..#\n" + 
			"  #.#. ... ..#\n" + 
			"### #.   \\ ..#\n" + 
			"#. .#..... **#######\n" + 
			"#.#\\#..... ..\\\\\\*. #\n" + 
			"#*\\\\#.###. ####\\\\\\ #\n" + 
			"#\\\\.#.     ...## \\ #\n" + 
			"#\\#.#..... ....# \\ #  \n" + 
			"###.#..... ....#   ##\n" + 
			"#\\\\.#..... ....#\\   # \n" + 
			"########.. ..###*####\n" + 
			"#......... .........#\n" + 
			"#......... ....***..#\n" + 
			"#..\\\\\\\\\\ # ####.....#\n" + 
			"#........*R..\\\\\\   .#\n" + 
			"##########L##########"
			),
	/**
	 * <pre>
		        #L#######
		        #*** \\ #
		        #\\\ .. #
		#########.##    ##########
		#.......\ ..........*   .#
		#*******\......#....#\\ .#
		###\.\\\...**..#....... *#
		#*****\\  .\\..##     #\.#
		######### ....  ##########
		        #       #
		        ####*####      
		        #.......#
		#########  \\\\*##########
		#*\\  **#     *..*\ \\\\\#
		#.\**\*** .....**.# \\##\#
		#\R......     .\\.. \\\\\#
		##########################
	 * </pre>
	 */
	MAP_09(
			"        #L#######\n" + 
			"        #*** \\\\ #\n" + 
			"        #\\\\\\ .. #\n" + 
			"#########.##    ##########\n" + 
			"#.......\\ ..........*   .#\n" + 
			"#*******\\......#....#\\\\ .#\n" + 
			"###\\.\\\\\\...**..#....... *#\n" + 
			"#*****\\\\  .\\\\..##     #\\.#\n" + 
			"######### ....  ##########\n" + 
			"        #       #\n" + 
			"        ####*####      \n" + 
			"        #.......#\n" + 
			"#########  \\\\\\\\*##########\n" + 
			"#*\\\\  **#     *..*\\ \\\\\\\\\\#\n" + 
			"#.\\**\\*** .....**.# \\\\##\\#\n" + 
			"#\\R......     .\\\\.. \\\\\\\\\\#\n" + 
			"##########################"
			),
	/**
	 * <pre>
		#############################
		#..........................\#
		#..\\###...#....        ###.#
		#..\*\\\.. #.... ..##\\..\#.#
		#..\*\.... #.... ..#\#....#.#
		#...\###.. #.... ....#....#.#
		#... ..... ..... .####......#
		#\\. #....           .......#
		#... #..#. .....*\ ##.......#
		#.#....... ...#..  ....######
		#. ...#... ...#.\  ....#..* #
		##........ ...#.. #....#.#\\#
		#.....*... .....*\#\\.....*.#
		#.***.* .......*\****.....#.#
		#.\\\.. ................   .#
		#.#####    .######    ##### #
		#....\\.................... #
		#....****...#.##.....\\\\..\#
		#....\\\\...#.........*....\#
		#....\\\\...#.\\.    #\###.\#
		#....     ..#.... ...#\\\\. #
		#........ ..#.... ...#..... #
		#........         ........#R#
		###########################L#
	 * </pre>
	 */
	MAP_10(
			"#############################\n" + 
			"#..........................\\#\n" + 
			"#..\\\\###...#....        ###.#\n" + 
			"#..\\*\\\\\\.. #.... ..##\\\\..\\#.#\n" + 
			"#..\\*\\.... #.... ..#\\#....#.#\n" + 
			"#...\\###.. #.... ....#....#.#\n" + 
			"#... ..... ..... .####......#\n" + 
			"#\\\\. #....           .......#\n" + 
			"#... #..#. .....*\\ ##.......#\n" + 
			"#.#....... ...#..  ....######\n" + 
			"#. ...#... ...#.\\  ....#..* #\n" + 
			"##........ ...#.. #....#.#\\\\#\n" + 
			"#.....*... .....*\\#\\\\.....*.#\n" + 
			"#.***.* .......*\\****.....#.#\n" + 
			"#.\\\\\\.. ................   .#\n" + 
			"#.#####    .######    ##### #\n" + 
			"#....\\\\.................... #\n" + 
			"#....****...#.##.....\\\\\\\\..\\#\n" + 
			"#....\\\\\\\\...#.........*....\\#\n" + 
			"#....\\\\\\\\...#.\\\\.    #\\###.\\#\n" + 
			"#....     ..#.... ...#\\\\\\\\. #\n" + 
			"#........ ..#.... ...#..... #\n" + 
			"#........         ........#R#\n" + 
			"###########################L#"
			),
	/**
	 * <pre>
		#################
		#A       .@@@@@@#
		#  ..    #*...*1#
		#  .##########*##
		#  ..@   \\\\#* #
		#  ..#   #####*.#
		#   .#####    * #
		#LR .......   . #
		#################
		
		Trampoline A targets 1
		Flooding 10
		Water 0
	 * </pre>
	 */
	HOROCK_2(
			"#################\n" + 
			"#A       .@@@@@@#\n" + 
			"#  ..    #*...*1#\n" + 
			"#  .##########*##\n" + 
			"#  ..@   \\\\\\\\#* #\n" + 
			"#  ..#   #####*.#\n" + 
			"#   .#####    * #\n" + 
			"#LR .......   . #\n" + 
			"#################\n" + 
			"\n" + 
			"Trampoline A targets 1\n" + 
			"Flooding 10\n" + 
			"Water 0"
			),
	/**
	 * <pre>
		###############
		#***...R......#
		#***... ...*..#
		#\\\... ..\\\.#
		#...... ...*..#
		#..     .. ...#
		#.... .... ...#
		#.... .... ...#
		#.. .       ..#
		#..*. .. .....#
		#.... .. .....#
		#.\.. .......*#
		#.............#
		#.........   .#
		#############L#
	 * </pre>
	 */
	MAP_FROM_TASK(
			"###############\n" + 
			"#***...R......#\n" + 
			"#***... ...*..#\n" + 
			"#\\\\\\... ..\\\\\\.#\n" + 
			"#...... ...*..#\n" + 
			"#..     .. ...#\n" + 
			"#.... .... ...#\n" + 
			"#.... .... ...#\n" + 
			"#.. .       ..#\n" + 
			"#..*. .. .....#\n" + 
			"#.... .. .....#\n" + 
			"#.\\.. .......*#\n" + 
			"#.............#\n" + 
			"#.........   .#\n" + 
			"#############L#"
			),
	/**
	 * <pre>
			##L###########
			#.....R#.**..#
			#.*A...#..1..#
			#.*....#.  \.#
			#.\\\..#...\.#
			#2.....**B...#
			##############
			
			Trampoline A targets 1
			Trampoline B targets 2
	 * </pre>
	 */
	MAP_FROM_TRAMPOLINES_TASK(
			"##L###########\n" + 
			"#.....R#.**..#\n" + 
			"#.*A...#..1..#\n" + 
			"#.*....#.  \\.#\n" + 
			"#.\\\\\\..#...\\.#\n" + 
			"#2.....**B...#\n" + 
			"##############\n" + 
			"\n" + 
			"Trampoline A targets 1\n" + 
			"Trampoline B targets 2"
			),
	/**
	 * <pre>
		###############
		#***...R......#
		#***... ...*..#
		#\\\!.. ..\\\.#
		#...... ...*..#
		#..     .. ...#
		#.... .... ...#
		#.... .... ...#
		#.. .  !    ..#
		#..*. .. . ...#
		#.... .. . ...#
		#.\.. .... ..*#
		#......... . .#
		#!....... W   #
		#############L#
		
		Growth 25
		Razors 1
	 * </pre>
	 */
	MAP_FROM_BEARDS_AND_RAZORS_TASK(
			"###############\n" + 
			"#***...R......#\n" + 
			"#***... ...*..#\n" + 
			"#\\\\\\!.. ..\\\\\\.#\n" + 
			"#...... ...*..#\n" + 
			"#..     .. ...#\n" + 
			"#.... .... ...#\n" + 
			"#.... .... ...#\n" + 
			"#.. .  !    ..#\n" + 
			"#..*. .. . ...#\n" + 
			"#.... .. . ...#\n" + 
			"#.\\.. .... ..*#\n" + 
			"#......... . .#\n" + 
			"#!....... W   #\n" + 
			"#############L#\n" + 
			"\n" + 
			"Growth 25\n" + 
			"Razors 1"
			);

	private String	map;

	private ExampleMaps(String map) {
		this.map = map;
	}

	public String getMap() {
		return map;
	}
}
